"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiskCache = void 0;
const cacache_1 = __importDefault(require("cacache"));
const moment_1 = __importDefault(require("moment"));
const autobind_decorator_1 = require("autobind-decorator");
const lodash_1 = require("lodash");
const MAX_AGE_IN_SECS = 6 * 60 * 60;
class DiskCache {
    async clear() {
        await cacache_1.default.rm.all('./tmp/');
    }
    async get(key, _options) {
        try {
            const { data, expire } = JSON.parse((await cacache_1.default.get('./tmp/', key)).data.toString());
            if (moment_1.default().valueOf() > expire) {
                return null;
            }
            else {
                return data;
            }
        }
        catch (error) {
            return null;
        }
    }
    async set(key, data, param3, param4) {
        let tags = [];
        let ttlInSecs = MAX_AGE_IN_SECS;
        if (Array.isArray(param3)) {
            tags = param3;
            if (typeof param4 === 'object' && param4 !== null) {
                ttlInSecs = lodash_1.get(param4, 'ttlInSecs', MAX_AGE_IN_SECS);
            }
        }
        else if (typeof param3 === 'object' && param3 !== null) {
            ttlInSecs = lodash_1.get(param3, 'ttlInSecs', MAX_AGE_IN_SECS);
        }
        const dataToCache = JSON.stringify({ data, expire: moment_1.default().add(ttlInSecs, 'seconds').valueOf() });
        await cacache_1.default.put('./tmp/', key, dataToCache);
        for (const tag of tags) {
            const encodedTag = `x-cache-tag/${tag}`;
            const tagData = (await this.get(encodedTag)) || [];
            if (!tagData.includes(key))
                tagData.push(key);
            const tagDataToCache = JSON.stringify({
                data: tagData,
                expire: moment_1.default()
                    .add(365 * 24 * 60 * 60, 'seconds')
                    .valueOf(),
            });
            await cacache_1.default.put('./tmp/', encodedTag, tagDataToCache);
        }
    }
    async delete(key, _options) {
        await this.set(key, null);
    }
    async invalidateTags(tags, _options) {
        for (const tag of tags) {
            const encodedTag = `x-cache-tag/${tag}`;
            const data = (await this.get(encodedTag)) || [];
            for (const key of data) {
                await this.delete(key);
            }
        }
    }
}
__decorate([
    autobind_decorator_1.boundMethod
], DiskCache.prototype, "clear", null);
__decorate([
    autobind_decorator_1.boundMethod
], DiskCache.prototype, "get", null);
__decorate([
    autobind_decorator_1.boundMethod
], DiskCache.prototype, "set", null);
__decorate([
    autobind_decorator_1.boundMethod
], DiskCache.prototype, "delete", null);
__decorate([
    autobind_decorator_1.boundMethod
], DiskCache.prototype, "invalidateTags", null);
exports.DiskCache = DiskCache;
