"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StructuredLogger = void 0;
const winston_1 = __importDefault(require("winston"));
const autobind_decorator_1 = require("autobind-decorator");
const { combine, timestamp: setTimestamp, printf } = winston_1.default.format;
class StructuredLogger {
    constructor(requestContextRegistry, indexablePropertiesOverride) {
        this.requestContextRegistry = requestContextRegistry;
        this.indexablePropertiesOverride = indexablePropertiesOverride;
        const indexableProperties = this.indexablePropertiesOverride || {
            type: 'string',
            action: 'string',
            label: 'string',
            category: 'string',
            value: 'number',
            durationInMillis: 'number',
            statusCode: 'string',
            code: 'string',
            namespace: 'string',
            request: 'string',
            error: 'string',
            errorCode: 'string',
            name: 'string',
            operator: 'string',
            function: 'string',
            module: 'string',
            path: 'string',
            details: 'string',
            title: 'string',
            field: 'string',
            resolver: 'string',
            customId: 'string',
            customKey: 'string',
            customIsoDatetime: 'string',
            matchedPath: 'string',
            matchedMethod: 'string',
        };
        this.logger = winston_1.default.createLogger({
            format: combine(setTimestamp(), printf(({ timestamp, level, message: rawMessage }) => {
                const message = rawMessage && typeof rawMessage === 'object' ? JSON.stringify(rawMessage) : String(rawMessage);
                const indexedFields = this.indexLogProperties(rawMessage, indexableProperties);
                const { requestId, authMethod, identitySummary, sourceIp, method, path, } = this.requestContextRegistry.getContext();
                return JSON.stringify({
                    timestamp,
                    level,
                    message,
                    requestId,
                    authMethod,
                    identitySummary,
                    method,
                    path,
                    sourceIp,
                    ...indexedFields,
                });
            })),
            transports: [new winston_1.default.transports.Console()],
        });
    }
    indexLogProperties(message, indexableProperties) {
        const indexedProperties = {};
        if (!message || typeof message !== 'object')
            return indexedProperties;
        for (const [key, type] of Object.entries(indexableProperties)) {
            if (!(key in message))
                continue;
            if (type === 'string' && ['string', 'number', 'boolean'].includes(typeof message[key])) {
                indexedProperties[key] = String(message[key]);
            }
            if (type === 'number' && typeof message[key] === 'number') {
                indexedProperties[key] = message[key];
            }
        }
        return indexedProperties;
    }
    log(log) {
        this.logger.log(log);
    }
    info(log) {
        this.logger.info(log);
    }
    debug(log) {
        this.logger.debug(log);
    }
    warn(log) {
        this.logger.warn(log);
    }
    error(log) {
        this.logger.error(log);
    }
}
__decorate([
    autobind_decorator_1.boundMethod
], StructuredLogger.prototype, "log", null);
__decorate([
    autobind_decorator_1.boundMethod
], StructuredLogger.prototype, "info", null);
__decorate([
    autobind_decorator_1.boundMethod
], StructuredLogger.prototype, "debug", null);
__decorate([
    autobind_decorator_1.boundMethod
], StructuredLogger.prototype, "warn", null);
__decorate([
    autobind_decorator_1.boundMethod
], StructuredLogger.prototype, "error", null);
exports.StructuredLogger = StructuredLogger;
