"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LambdaRouter = void 0;
const koa_1 = __importDefault(require("koa"));
const router_1 = __importDefault(require("@koa/router"));
const serverless_http_1 = __importDefault(require("serverless-http"));
const koa_bodyparser_1 = __importDefault(require("koa-bodyparser"));
const autobind_decorator_1 = require("autobind-decorator");
const cors_1 = __importDefault(require("@koa/cors"));
class LambdaRouter {
    constructor(logger, routes, corsConfig) {
        this.logger = logger;
        const app = new koa_1.default();
        this.router = new router_1.default();
        this.addRoutes(routes);
        if (corsConfig?.enabled) {
            const { config } = corsConfig;
            app.use(cors_1.default(config));
        }
        app
            .use(koa_bodyparser_1.default({
            enableTypes: ['json', 'xml', 'form'],
            jsonLimit: '10mb',
            xmlLimit: '10mb',
            formLimit: '10mb',
        }))
            .use(async (ctx, next) => {
            const contentTypeHeader = Object.entries(ctx.request.headers).find(([key]) => key.toLowerCase() === 'content-type');
            const contentType = !contentTypeHeader
                ? undefined
                : Array.isArray(contentTypeHeader[1])
                    ? contentTypeHeader[1][0]
                    : contentTypeHeader[1];
            if (contentType &&
                !['application/json', 'text/xml', 'application/xml', 'application/x-www-form-urlencoded'].includes(contentType)) {
                ctx.status = 415;
                ctx.body = 'Unsupported media type';
                return;
            }
            if (ctx.request.rawBody === undefined) {
                ctx.request.body = undefined;
            }
            await next();
        })
            .use(this.router.routes())
            .use(this.router.allowedMethods());
        this.handler = serverless_http_1.default(app);
    }
    async route(event, context) {
        const result = await this.handler(event, context);
        return result;
    }
    addRoutes(routes) {
        for (const route of routes) {
            const { path, get, post, put, patch, delete: del } = route;
            if (get)
                this.router.get(path, this.convertToMiddleware(get));
            if (post)
                this.router.post(path, this.convertToMiddleware(post));
            if (put)
                this.router.put(path, this.convertToMiddleware(put));
            if (patch)
                this.router.patch(path, this.convertToMiddleware(patch));
            if (del)
                this.router.delete(path, this.convertToMiddleware(del));
        }
        return this;
    }
    convertToMiddleware(routeConfig) {
        const { handler, tracingConfig: { logRequest, logResponse }, matchedMethod, matchedPath, } = routeConfig;
        return async (ctx, next) => {
            const multiValueQuery = Object.entries(ctx.query ?? {}).reduce((res, [k, v]) => ({
                ...res,
                [k]: Array.isArray(v) ? v : [v],
            }), {});
            const query = Object.entries(multiValueQuery).reduce((res, [k, v]) => ({
                ...res,
                [k]: v[0],
            }), {});
            const headers = Object.entries(ctx.headers ?? {}).reduce((res, [k, v]) => ({
                ...res,
                [k.toLowerCase()]: v,
            }), {});
            headers.getHeader = (header) => {
                return headers[header.toLowerCase()];
            };
            const request = {
                path: ctx.params ?? {},
                body: ctx.request.body,
                query,
                headers,
                multiValueQuery,
                queryString: ctx.querystring ?? '',
            };
            if (logRequest)
                this.logger.info({
                    category: 'TRACING',
                    label: 'REQUEST',
                    matchedMethod,
                    matchedPath,
                    data: request,
                });
            const response = await handler(request);
            if (logResponse)
                this.logger.info({
                    category: 'TRACING',
                    label: 'RESPONSE',
                    matchedMethod,
                    matchedPath,
                    data: response,
                    statusCode: response.statusCode,
                });
            if (response.body)
                ctx.body = response.body;
            ctx.status = response.statusCode;
            if (response.headers)
                ctx.response.set(response.headers);
            await next();
        };
    }
}
__decorate([
    autobind_decorator_1.boundMethod
], LambdaRouter.prototype, "route", null);
__decorate([
    autobind_decorator_1.boundMethod
], LambdaRouter.prototype, "addRoutes", null);
exports.LambdaRouter = LambdaRouter;
